@@ -2,16 +2,19 @@
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
+from __future__ import annotations
+
 import os
 import sys
-from typing import Optional, Sequence
+from collections.abc import Sequence
+from typing import NoReturn
 
 from pylint.__pkginfo__ import __version__
 
 # pylint: disable=import-outside-toplevel
 
 
-def run_pylint(argv: Optional[Sequence[str]] = None):
+def run_pylint(argv: Sequence[str] | None = None) -> None:
     """Run pylint.
 
     argv can be a sequence of strings normally supplied as arguments on the command line
@@ -24,7 +27,17 @@ def run_pylint(argv: Optional[Sequence[str]] = None):
         sys.exit(1)
 
 
-def run_epylint(argv: Optional[Sequence[str]] = None):
+def _run_pylint_config(argv: Sequence[str] | None = None) -> None:
+    """Run pylint-config.
+
+    argv can be a sequence of strings normally supplied as arguments on the command line
+    """
+    from pylint.lint.run import _PylintConfigRun
+
+    _PylintConfigRun(argv or sys.argv[1:])
+
+
+def run_epylint(argv: Sequence[str] | None = None) -> NoReturn:
     """Run epylint.
 
     argv can be a list of strings normally supplied as arguments on the command line
@@ -34,7 +47,7 @@ def run_epylint(argv: Optional[Sequence[str]] = None):
     EpylintRun(argv)
 
 
-def run_pyreverse(argv: Optional[Sequence[str]] = None):
+def run_pyreverse(argv: Sequence[str] | None = None) -> NoReturn:  # type: ignore[misc]
     """Run pyreverse.
 
     argv can be a sequence of strings normally supplied as arguments on the command line
@@ -44,7 +57,7 @@ def run_pyreverse(argv: Optional[Sequence[str]] = None):
     PyreverseRun(argv or sys.argv[1:])
 
 
-def run_symilar(argv: Optional[Sequence[str]] = None):
+def run_symilar(argv: Sequence[str] | None = None) -> NoReturn:
     """Run symilar.
 
     argv can be a sequence of strings normally supplied as arguments on the command line
