@@ -4,24 +4,25 @@
 
 """Basic checker for Python code."""
 
+from __future__ import annotations
+
 import collections
 import itertools
 import sys
-from typing import TYPE_CHECKING, Dict, Optional, cast
+from collections.abc import Iterator
+from typing import TYPE_CHECKING, cast
 
 import astroid
 from astroid import nodes
 
-from pylint import interfaces
 from pylint import utils as lint_utils
 from pylint.checkers import BaseChecker, utils
-from pylint.interfaces import IAstroidChecker
+from pylint.interfaces import HIGH
 from pylint.reporters.ureports import nodes as reporter_nodes
 from pylint.utils import LinterStats
-from pylint.utils.utils import get_global_option
 
 if TYPE_CHECKING:
-    pass
+    from pylint.lint.pylinter import PyLinter
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -30,9 +31,10 @@ else:
 
 
 class _BasicChecker(BaseChecker):
-    """Permits separating multiple checks with the same checker name into classes/file."""
+    """Permits separating multiple checks with the same checker name into
+    classes/file.
+    """
 
-    __implements__ = IAstroidChecker
     name = "basic"
 
 
@@ -62,17 +64,17 @@ DEFAULT_ARGUMENT_SYMBOLS = dict(
 
 
 def report_by_type_stats(
-    sect,
+    sect: reporter_nodes.Section,
     stats: LinterStats,
-    old_stats: Optional[LinterStats],
-):
+    old_stats: LinterStats | None,
+) -> None:
     """Make a report of.
 
     * percentage of different types documented
     * percentage of different types with a bad name
     """
     # percentage of different types documented and/or with a bad name
-    nice_stats: Dict[str, Dict[str, str]] = {}
+    nice_stats: dict[str, dict[str, str]] = {}
     for node_type in ("module", "class", "method", "function"):
         node_type = cast(Literal["function", "class", "method", "module"], node_type)
         total = stats.get_node_count(node_type)
@@ -115,8 +117,6 @@ class BasicChecker(_BasicChecker):
     * uses of the global statement
     """
 
-    __implements__ = interfaces.IAstroidChecker
-
     name = "basic"
     msgs = {
         "W0101": (
@@ -246,36 +246,51 @@ class BasicChecker(_BasicChecker):
             "Used when an assert statement has a string literal as its first argument, which will "
             "cause the assert to always pass.",
         ),
+        "W0130": (
+            "Duplicate value %r in set",
+            "duplicate-value",
+            "This message is emitted when a set contains the same value two or more times.",
+        ),
     }
 
     reports = (("RP0101", "Statistics by type", report_by_type_stats),)
 
-    def __init__(self, linter):
+    def __init__(self, linter: PyLinter) -> None:
         super().__init__(linter)
-        self._tryfinallys = None
+        self._tryfinallys: list[nodes.TryFinally] | None = None
 
-    def open(self):
+    def open(self) -> None:
         """Initialize visit variables and statistics."""
-        py_version = get_global_option(self, "py-version")
+        py_version = self.linter.config.py_version
         self._py38_plus = py_version >= (3, 8)
         self._tryfinallys = []
         self.linter.stats.reset_node_count()
 
-    @utils.check_messages("using-constant-test", "missing-parentheses-for-call-in-test")
+    @utils.only_required_for_messages(
+        "using-constant-test", "missing-parentheses-for-call-in-test"
+    )
     def visit_if(self, node: nodes.If) -> None:
         self._check_using_constant_test(node, node.test)
 
-    @utils.check_messages("using-constant-test", "missing-parentheses-for-call-in-test")
+    @utils.only_required_for_messages(
+        "using-constant-test", "missing-parentheses-for-call-in-test"
+    )
     def visit_ifexp(self, node: nodes.IfExp) -> None:
         self._check_using_constant_test(node, node.test)
 
-    @utils.check_messages("using-constant-test", "missing-parentheses-for-call-in-test")
+    @utils.only_required_for_messages(
+        "using-constant-test", "missing-parentheses-for-call-in-test"
+    )
     def visit_comprehension(self, node: nodes.Comprehension) -> None:
         if node.ifs:
             for if_test in node.ifs:
                 self._check_using_constant_test(node, if_test)
 
-    def _check_using_constant_test(self, node, test):
+    def _check_using_constant_test(
+        self,
+        node: nodes.If | nodes.IfExp | nodes.Comprehension,
+        test: nodes.NodeNG | None,
+    ) -> None:
         const_nodes = (
             nodes.Module,
             nodes.GeneratorExp,
@@ -304,7 +319,7 @@ class BasicChecker(_BasicChecker):
             inferred = utils.safe_infer(test)
 
         if emit:
-            self.add_message("using-constant-test", node=node)
+            self.add_message("using-constant-test", node=test)
         elif isinstance(inferred, const_nodes):
             # If the constant node is a FunctionDef or Lambda then
             # it may be an illicit function call due to missing parentheses
@@ -321,12 +336,12 @@ class BasicChecker(_BasicChecker):
                     for inf_call in call_inferred:
                         if inf_call != astroid.Uninferable:
                             self.add_message(
-                                "missing-parentheses-for-call-in-test", node=node
+                                "missing-parentheses-for-call-in-test", node=test
                             )
                             break
                 except astroid.InferenceError:
                     pass
-            self.add_message("using-constant-test", node=node)
+            self.add_message("using-constant-test", node=test)
 
     def visit_module(self, _: nodes.Module) -> None:
         """Check module name, docstring and required arguments."""
@@ -334,11 +349,11 @@ class BasicChecker(_BasicChecker):
 
     def visit_classdef(self, _: nodes.ClassDef) -> None:
         """Check module name, docstring and redefinition
-        increment branch counter
+        increment branch counter.
         """
         self.linter.stats.node_count["klass"] += 1
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "pointless-statement", "pointless-string-statement", "expression-not-assigned"
     )
     def visit_expr(self, node: nodes.Expr) -> None:
@@ -385,7 +400,9 @@ class BasicChecker(_BasicChecker):
             self.add_message("pointless-statement", node=node)
 
     @staticmethod
-    def _filter_vararg(node, call_args):
+    def _filter_vararg(
+        node: nodes.Lambda, call_args: list[nodes.NodeNG]
+    ) -> Iterator[nodes.NodeNG]:
         # Return the arguments for the given call which are
         # not passed as vararg.
         for arg in call_args:
@@ -399,18 +416,17 @@ class BasicChecker(_BasicChecker):
                 yield arg
 
     @staticmethod
-    def _has_variadic_argument(args, variadic_name):
-        if not args:
-            return True
-        for arg in args:
-            if isinstance(arg.value, nodes.Name):
-                if arg.value.name != variadic_name:
-                    return True
-            else:
-                return True
-        return False
+    def _has_variadic_argument(
+        args: list[nodes.Starred | nodes.Keyword], variadic_name: str
+    ) -> bool:
+        return not args or any(
+            isinstance(a.value, nodes.Name)
+            and a.value.name != variadic_name
+            or not isinstance(a.value, nodes.Name)
+            for a in args
+        )
 
-    @utils.check_messages("unnecessary-lambda")
+    @utils.only_required_for_messages("unnecessary-lambda")
     def visit_lambda(self, node: nodes.Lambda) -> None:
         """Check whether the lambda is suspicious."""
         # if the body of the lambda is a call expression with the same
@@ -470,10 +486,10 @@ class BasicChecker(_BasicChecker):
 
         self.add_message("unnecessary-lambda", line=node.fromlineno, node=node)
 
-    @utils.check_messages("dangerous-default-value")
+    @utils.only_required_for_messages("dangerous-default-value")
     def visit_functiondef(self, node: nodes.FunctionDef) -> None:
         """Check function name, docstring, arguments, redefinition,
-        variable names, max locals
+        variable names, max locals.
         """
         if node.is_method():
             self.linter.stats.node_count["method"] += 1
@@ -483,10 +499,10 @@ class BasicChecker(_BasicChecker):
 
     visit_asyncfunctiondef = visit_functiondef
 
-    def _check_dangerous_default(self, node):
+    def _check_dangerous_default(self, node: nodes.FunctionDef) -> None:
         """Check for dangerous default values as arguments."""
 
-        def is_iterable(internal_node):
+        def is_iterable(internal_node: nodes.NodeNG) -> bool:
             return isinstance(internal_node, (nodes.List, nodes.Set, nodes.Dict))
 
         defaults = node.args.defaults or [] + node.args.kw_defaults or []
@@ -523,7 +539,7 @@ class BasicChecker(_BasicChecker):
                     msg = f"{default.as_string()} ({DEFAULT_ARGUMENT_SYMBOLS[value.qname()]})"
                 self.add_message("dangerous-default-value", node=node, args=(msg,))
 
-    @utils.check_messages("unreachable", "lost-exception")
+    @utils.only_required_for_messages("unreachable", "lost-exception")
     def visit_return(self, node: nodes.Return) -> None:
         """Return node visitor.
 
@@ -536,14 +552,14 @@ class BasicChecker(_BasicChecker):
         # Is it inside final body of a try...finally block ?
         self._check_not_in_finally(node, "return", (nodes.FunctionDef,))
 
-    @utils.check_messages("unreachable")
+    @utils.only_required_for_messages("unreachable")
     def visit_continue(self, node: nodes.Continue) -> None:
         """Check is the node has a right sibling (if so, that's some unreachable
-        code)
+        code).
         """
         self._check_unreachable(node)
 
-    @utils.check_messages("unreachable", "lost-exception")
+    @utils.only_required_for_messages("unreachable", "lost-exception")
     def visit_break(self, node: nodes.Break) -> None:
         """Break node visitor.
 
@@ -557,14 +573,14 @@ class BasicChecker(_BasicChecker):
         # 2 - Is it inside final body of a try...finally block ?
         self._check_not_in_finally(node, "break", (nodes.For, nodes.While))
 
-    @utils.check_messages("unreachable")
+    @utils.only_required_for_messages("unreachable")
     def visit_raise(self, node: nodes.Raise) -> None:
         """Check if the node has a right sibling (if so, that's some unreachable
-        code)
+        code).
         """
         self._check_unreachable(node)
 
-    def _check_misplaced_format_function(self, call_node):
+    def _check_misplaced_format_function(self, call_node: nodes.Call) -> None:
         if not isinstance(call_node.func, nodes.Attribute):
             return
         if call_node.func.attrname != "format":
@@ -585,12 +601,12 @@ class BasicChecker(_BasicChecker):
             ):
                 self.add_message("misplaced-format-function", node=call_node)
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "eval-used", "exec-used", "bad-reversed-sequence", "misplaced-format-function"
     )
     def visit_call(self, node: nodes.Call) -> None:
         """Visit a Call node -> check if this is not a disallowed builtin
-        call and check for * or ** use
+        call and check for * or ** use.
         """
         self._check_misplaced_format_function(node)
         if isinstance(node.func, nodes.Name):
@@ -605,7 +621,7 @@ class BasicChecker(_BasicChecker):
                 elif name == "eval":
                     self.add_message("eval-used", node=node)
 
-    @utils.check_messages("assert-on-tuple", "assert-on-string-literal")
+    @utils.only_required_for_messages("assert-on-tuple", "assert-on-string-literal")
     def visit_assert(self, node: nodes.Assert) -> None:
         """Check whether assert is used on a tuple or string literal."""
         if (
@@ -622,7 +638,7 @@ class BasicChecker(_BasicChecker):
                 when = "always"
             self.add_message("assert-on-string-literal", node=node, args=(when,))
 
-    @utils.check_messages("duplicate-key")
+    @utils.only_required_for_messages("duplicate-key")
     def visit_dict(self, node: nodes.Dict) -> None:
         """Check duplicate key in dictionary."""
         keys = set()
@@ -637,15 +653,34 @@ class BasicChecker(_BasicChecker):
                 self.add_message("duplicate-key", node=node, args=key)
             keys.add(key)
 
+    @utils.only_required_for_messages("duplicate-value")
+    def visit_set(self, node: nodes.Set) -> None:
+        """Check duplicate value in set."""
+        values = set()
+        for v in node.elts:
+            if isinstance(v, nodes.Const):
+                value = v.value
+            else:
+                continue
+            if value in values:
+                self.add_message(
+                    "duplicate-value", node=node, args=value, confidence=HIGH
+                )
+            values.add(value)
+
     def visit_tryfinally(self, node: nodes.TryFinally) -> None:
         """Update try...finally flag."""
+        assert self._tryfinallys is not None
         self._tryfinallys.append(node)
 
     def leave_tryfinally(self, _: nodes.TryFinally) -> None:
         """Update try...finally flag."""
+        assert self._tryfinallys is not None
         self._tryfinallys.pop()
 
-    def _check_unreachable(self, node):
+    def _check_unreachable(
+        self, node: nodes.Return | nodes.Continue | nodes.Break | nodes.Raise
+    ) -> None:
         """Check unreachable code."""
         unreach_stmt = node.next_sibling()
         if unreach_stmt is not None:
@@ -661,7 +696,12 @@ class BasicChecker(_BasicChecker):
                     return
             self.add_message("unreachable", node=unreach_stmt)
 
-    def _check_not_in_finally(self, node, node_name, breaker_classes=()):
+    def _check_not_in_finally(
+        self,
+        node: nodes.Break | nodes.Return,
+        node_name: str,
+        breaker_classes: tuple[nodes.NodeNG, ...] = (),
+    ) -> None:
         """Check that a node is not inside a 'finally' clause of a
         'try...finally' statement.
 
@@ -681,7 +721,7 @@ class BasicChecker(_BasicChecker):
             _node = _parent
             _parent = _node.parent
 
-    def _check_reversed(self, node):
+    def _check_reversed(self, node: nodes.Call) -> None:
         """Check that the argument to `reversed` is a sequence."""
         try:
             argument = utils.safe_infer(utils.get_argument_from_call(node, position=0))
@@ -707,7 +747,7 @@ class BasicChecker(_BasicChecker):
             if isinstance(argument, (nodes.List, nodes.Tuple)):
                 return
 
-            # dicts are reversible, but only from Python 3.8 onwards. Prior to
+            # dicts are reversible, but only from Python 3.8 onward. Prior to
             # that, any class based on dict must explicitly provide a
             # __reversed__ method
             if not self._py38_plus and isinstance(argument, astroid.Instance):
@@ -738,7 +778,7 @@ class BasicChecker(_BasicChecker):
             else:
                 self.add_message("bad-reversed-sequence", node=node)
 
-    @utils.check_messages("confusing-with-statement")
+    @utils.only_required_for_messages("confusing-with-statement")
     def visit_with(self, node: nodes.With) -> None:
         # a "with" statement with multiple managers corresponds
         # to one AST "With" node with multiple items
@@ -754,7 +794,7 @@ class BasicChecker(_BasicChecker):
                     # we assume it's a nested "with".
                     self.add_message("confusing-with-statement", node=node)
 
-    def _check_self_assigning_variable(self, node):
+    def _check_self_assigning_variable(self, node: nodes.Assign) -> None:
         # Detect assigning to the same variable.
 
         scope = node.scope()
@@ -795,10 +835,8 @@ class BasicChecker(_BasicChecker):
                     "self-assigning-variable", args=(target.name,), node=target
                 )
 
-    def _check_redeclared_assign_name(self, targets):
-        dummy_variables_rgx = lint_utils.get_global_option(
-            self, "dummy-variables-rgx", default=None
-        )
+    def _check_redeclared_assign_name(self, targets: list[nodes.NodeNG | None]) -> None:
+        dummy_variables_rgx = self.linter.config.dummy_variables_rgx
 
         for target in targets:
             if not isinstance(target, nodes.Tuple):
@@ -820,11 +858,13 @@ class BasicChecker(_BasicChecker):
                         "redeclared-assigned-name", args=(name,), node=target
                     )
 
-    @utils.check_messages("self-assigning-variable", "redeclared-assigned-name")
+    @utils.only_required_for_messages(
+        "self-assigning-variable", "redeclared-assigned-name"
+    )
     def visit_assign(self, node: nodes.Assign) -> None:
         self._check_self_assigning_variable(node)
         self._check_redeclared_assign_name(node.targets)
 
-    @utils.check_messages("redeclared-assigned-name")
+    @utils.only_required_for_messages("redeclared-assigned-name")
     def visit_for(self, node: nodes.For) -> None:
         self._check_redeclared_assign_name([node.target])
