@@ -4,17 +4,19 @@
 
 """Basic Error checker from the basic checker."""
 
-import collections
+from __future__ import annotations
+
 import itertools
-from typing import Any, Iterator
+from collections.abc import Iterator
+from typing import Any
 
 import astroid
 from astroid import nodes
 
-from pylint import utils as lint_utils
 from pylint.checkers import utils
 from pylint.checkers.base.basic_checker import _BasicChecker
 from pylint.checkers.utils import infer_all
+from pylint.interfaces import HIGH
 
 ABC_METACLASSES = {"_py_abc.ABCMeta", "abc.ABCMeta"}  # Python 3.7+,
 # List of methods which can be redefined
@@ -22,7 +24,7 @@ REDEFINABLE_METHODS = frozenset(("__module__",))
 TYPING_FORWARD_REF_QNAME = "typing.ForwardRef"
 
 
-def _get_break_loop_node(break_node):
+def _get_break_loop_node(break_node: nodes.Break) -> nodes.For | nodes.While | None:
     """Returns the loop node that holds the break node in arguments.
 
     Args:
@@ -43,7 +45,7 @@ def _get_break_loop_node(break_node):
     return parent
 
 
-def _loop_exits_early(loop):
+def _loop_exits_early(loop: nodes.For | nodes.While) -> bool:
     """Returns true if a loop may end with a break statement.
 
     Args:
@@ -54,7 +56,7 @@ def _loop_exits_early(loop):
     """
     loop_nodes = (nodes.For, nodes.While)
     definition_nodes = (nodes.FunctionDef, nodes.ClassDef)
-    inner_loop_nodes = [
+    inner_loop_nodes: list[nodes.For | nodes.While] = [
         _node
         for _node in loop.nodes_of_class(loop_nodes, skip_klass=definition_nodes)
         if _node != loop
@@ -75,7 +77,7 @@ def _has_abstract_methods(node):
     return len(utils.unimplemented_abstract_methods(node)) > 0
 
 
-def redefined_by_decorator(node):
+def redefined_by_decorator(node: nodes.FunctionDef) -> bool:
     """Return True if the object is a method redefined via decorator.
 
     For example:
@@ -204,11 +206,11 @@ class BasicErrorChecker(_BasicChecker):
         ),
     }
 
-    @utils.check_messages("function-redefined")
+    @utils.only_required_for_messages("function-redefined")
     def visit_classdef(self, node: nodes.ClassDef) -> None:
         self._check_redefinition("class", node)
 
-    def _too_many_starred_for_tuple(self, assign_tuple):
+    def _too_many_starred_for_tuple(self, assign_tuple: nodes.Tuple) -> bool:
         starred_count = 0
         for elem in assign_tuple.itered():
             if isinstance(elem, nodes.Tuple):
@@ -217,7 +219,9 @@ class BasicErrorChecker(_BasicChecker):
                 starred_count += 1
         return starred_count > 1
 
-    @utils.check_messages("too-many-star-expressions", "invalid-star-assignment-target")
+    @utils.only_required_for_messages(
+        "too-many-star-expressions", "invalid-star-assignment-target"
+    )
     def visit_assign(self, node: nodes.Assign) -> None:
         # Check *a, *b = ...
         assign_target = node.targets[0]
@@ -230,7 +234,7 @@ class BasicErrorChecker(_BasicChecker):
         if self._too_many_starred_for_tuple(assign_target):
             self.add_message("too-many-star-expressions", node=node)
 
-    @utils.check_messages("star-needs-assignment-target")
+    @utils.only_required_for_messages("star-needs-assignment-target")
     def visit_starred(self, node: nodes.Starred) -> None:
         """Check that a Starred expression is used in an assignment target."""
         if isinstance(node.parent, nodes.Call):
@@ -248,7 +252,7 @@ class BasicErrorChecker(_BasicChecker):
         if stmt.value is node or stmt.value.parent_of(node):
             self.add_message("star-needs-assignment-target", node=node)
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "init-is-generator",
         "return-in-init",
         "function-redefined",
@@ -277,26 +281,22 @@ class BasicErrorChecker(_BasicChecker):
                 if any(v for v in values if not utils.is_none(v)):
                     self.add_message("return-in-init", node=node)
         # Check for duplicate names by clustering args with same name for detailed report
-        arg_clusters = collections.defaultdict(list)
+        arg_clusters = {}
         arguments: Iterator[Any] = filter(None, [node.args.args, node.args.kwonlyargs])
-
         for arg in itertools.chain.from_iterable(arguments):
-            arg_clusters[arg.name].append(arg)
-
-        # provide detailed report about each repeated argument
-        for argument_duplicates in arg_clusters.values():
-            if len(argument_duplicates) != 1:
-                for argument in argument_duplicates:
-                    self.add_message(
-                        "duplicate-argument-name",
-                        line=argument.lineno,
-                        node=argument,
-                        args=(argument.name,),
-                    )
+            if arg.name in arg_clusters:
+                self.add_message(
+                    "duplicate-argument-name",
+                    node=arg,
+                    args=(arg.name,),
+                    confidence=HIGH,
+                )
+            else:
+                arg_clusters[arg.name] = arg
 
     visit_asyncfunctiondef = visit_functiondef
 
-    def _check_name_used_prior_global(self, node):
+    def _check_name_used_prior_global(self, node: nodes.FunctionDef) -> None:
 
         scope_globals = {
             name: child
@@ -323,10 +323,10 @@ class BasicErrorChecker(_BasicChecker):
                     "used-prior-global-declaration", node=node_name, args=(name,)
                 )
 
-    def _check_nonlocal_and_global(self, node):
+    def _check_nonlocal_and_global(self, node: nodes.FunctionDef) -> None:
         """Check that a name is both nonlocal and global."""
 
-        def same_scope(current):
+        def same_scope(current: nodes.Global | nodes.Nonlocal) -> bool:
             return current.scope() is node
 
         from_iter = itertools.chain.from_iterable
@@ -351,36 +351,36 @@ class BasicErrorChecker(_BasicChecker):
         for name in nonlocals.intersection(global_vars):
             self.add_message("nonlocal-and-global", args=(name,), node=node)
 
-    @utils.check_messages("return-outside-function")
+    @utils.only_required_for_messages("return-outside-function")
     def visit_return(self, node: nodes.Return) -> None:
         if not isinstance(node.frame(future=True), nodes.FunctionDef):
             self.add_message("return-outside-function", node=node)
 
-    @utils.check_messages("yield-outside-function")
+    @utils.only_required_for_messages("yield-outside-function")
     def visit_yield(self, node: nodes.Yield) -> None:
         self._check_yield_outside_func(node)
 
-    @utils.check_messages("yield-outside-function")
+    @utils.only_required_for_messages("yield-outside-function")
     def visit_yieldfrom(self, node: nodes.YieldFrom) -> None:
         self._check_yield_outside_func(node)
 
-    @utils.check_messages("not-in-loop", "continue-in-finally")
+    @utils.only_required_for_messages("not-in-loop", "continue-in-finally")
     def visit_continue(self, node: nodes.Continue) -> None:
         self._check_in_loop(node, "continue")
 
-    @utils.check_messages("not-in-loop")
+    @utils.only_required_for_messages("not-in-loop")
     def visit_break(self, node: nodes.Break) -> None:
         self._check_in_loop(node, "break")
 
-    @utils.check_messages("useless-else-on-loop")
+    @utils.only_required_for_messages("useless-else-on-loop")
     def visit_for(self, node: nodes.For) -> None:
         self._check_else_on_loop(node)
 
-    @utils.check_messages("useless-else-on-loop")
+    @utils.only_required_for_messages("useless-else-on-loop")
     def visit_while(self, node: nodes.While) -> None:
         self._check_else_on_loop(node)
 
-    @utils.check_messages("nonexistent-operator")
+    @utils.only_required_for_messages("nonexistent-operator")
     def visit_unaryop(self, node: nodes.UnaryOp) -> None:
         """Check use of the non-existent ++ and -- operators."""
         if (
@@ -391,7 +391,7 @@ class BasicErrorChecker(_BasicChecker):
         ):
             self.add_message("nonexistent-operator", node=node, args=node.op * 2)
 
-    def _check_nonlocal_without_binding(self, node, name):
+    def _check_nonlocal_without_binding(self, node: nodes.Nonlocal, name: str) -> None:
         current_scope = node.scope()
         while True:
             if current_scope.parent is None:
@@ -411,12 +411,12 @@ class BasicErrorChecker(_BasicChecker):
         if not isinstance(current_scope, nodes.FunctionDef):
             self.add_message("nonlocal-without-binding", args=(name,), node=node)
 
-    @utils.check_messages("nonlocal-without-binding")
+    @utils.only_required_for_messages("nonlocal-without-binding")
     def visit_nonlocal(self, node: nodes.Nonlocal) -> None:
         for name in node.names:
             self._check_nonlocal_without_binding(node, name)
 
-    @utils.check_messages("abstract-class-instantiated")
+    @utils.only_required_for_messages("abstract-class-instantiated")
     def visit_call(self, node: nodes.Call) -> None:
         """Check instantiating abstract class with
         abc.ABCMeta as metaclass.
@@ -424,7 +424,7 @@ class BasicErrorChecker(_BasicChecker):
         for inferred in infer_all(node.func):
             self._check_inferred_class_is_abstract(inferred, node)
 
-    def _check_inferred_class_is_abstract(self, inferred, node):
+    def _check_inferred_class_is_abstract(self, inferred, node: nodes.Call):
         if not isinstance(inferred, nodes.ClassDef):
             return
 
@@ -461,11 +461,11 @@ class BasicErrorChecker(_BasicChecker):
                 "abstract-class-instantiated", args=(inferred.name,), node=node
             )
 
-    def _check_yield_outside_func(self, node):
+    def _check_yield_outside_func(self, node: nodes.Yield) -> None:
         if not isinstance(node.frame(future=True), (nodes.FunctionDef, nodes.Lambda)):
             self.add_message("yield-outside-function", node=node)
 
-    def _check_else_on_loop(self, node):
+    def _check_else_on_loop(self, node: nodes.For | nodes.While) -> None:
         """Check that any loop with an else clause has a break statement."""
         if node.orelse and not _loop_exits_early(node):
             self.add_message(
@@ -477,7 +477,9 @@ class BasicErrorChecker(_BasicChecker):
                 line=node.orelse[0].lineno - 1,
             )
 
-    def _check_in_loop(self, node, node_name):
+    def _check_in_loop(
+        self, node: nodes.Continue | nodes.Break, node_name: str
+    ) -> None:
         """Check that a node is inside a for or while loop."""
         for parent in node.node_ancestors():
             if isinstance(parent, (nodes.For, nodes.While)):
@@ -495,7 +497,9 @@ class BasicErrorChecker(_BasicChecker):
 
         self.add_message("not-in-loop", node=node, args=node_name)
 
-    def _check_redefinition(self, redeftype, node):
+    def _check_redefinition(
+        self, redeftype: str, node: nodes.Call | nodes.FunctionDef
+    ) -> None:
         """Check for redefinition of a function / method / class name."""
         parent_frame = node.parent.frame(future=True)
 
@@ -560,9 +564,7 @@ class BasicErrorChecker(_BasicChecker):
                     ):
                         return
 
-            dummy_variables_rgx = lint_utils.get_global_option(
-                self, "dummy-variables-rgx", default=None
-            )
+            dummy_variables_rgx = self.linter.config.dummy_variables_rgx
             if dummy_variables_rgx and dummy_variables_rgx.match(node.name):
                 return
             self.add_message(
