@@ -4,17 +4,19 @@
 
 """Imports checkers for Python code."""
 
+from __future__ import annotations
+
 import collections
 import copy
 import os
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set, Tuple, Union
+import sys
+from typing import TYPE_CHECKING, Any
 
 import astroid
 from astroid import nodes
 
 from pylint.checkers import BaseChecker, DeprecatedMixin
 from pylint.checkers.utils import (
-    check_messages,
     get_import_name,
     is_from_fallback_block,
     is_node_in_guarded_import_block,
@@ -23,14 +25,50 @@ from pylint.checkers.utils import (
 )
 from pylint.exceptions import EmptyReportError
 from pylint.graph import DotBackend, get_cycles
-from pylint.interfaces import IAstroidChecker
 from pylint.reporters.ureports.nodes import Paragraph, Section, VerbatimText
-from pylint.utils import IsortDriver, get_global_option
+from pylint.typing import MessageDefinitionTuple
+from pylint.utils import IsortDriver
 
 if TYPE_CHECKING:
     from pylint.lint import PyLinter
 
 
+DEPRECATED_MODULES = {
+    (0, 0, 0): {"tkinter.tix", "fpectl"},
+    (3, 2, 0): {"optparse"},
+    (3, 3, 0): {"xml.etree.cElementTree"},
+    (3, 4, 0): {"imp"},
+    (3, 5, 0): {"formatter"},
+    (3, 6, 0): {"asynchat", "asyncore", "smtpd"},
+    (3, 7, 0): {"macpath"},
+    (3, 9, 0): {"lib2to3", "parser", "symbol", "binhex"},
+    (3, 10, 0): {"distutils"},
+    (3, 11, 0): {
+        "aifc",
+        "audioop",
+        "cgi",
+        "cgitb",
+        "chunk",
+        "crypt",
+        "imghdr",
+        "msilib",
+        "nis",
+        "nntplib",
+        "ossaudiodev",
+        "pipes",
+        "sndhdr",
+        "spwd",
+        "sunau",
+        "sre_compile",
+        "sre_constants",
+        "sre_parse",
+        "telnetlib",
+        "uu",
+        "xdrlib",
+    },
+}
+
+
 def _qualified_names(modname):
     """Split the names of the given module into subparts.
 
@@ -96,7 +134,7 @@ def _ignore_import_failure(node, modname, ignored_modules):
 
 def _make_tree_defs(mod_files_list):
     """Get a list of 2-uple (module, list_of_files_which_import_this_module),
-    it will return a dictionary to represent this as a tree
+    it will return a dictionary to represent this as a tree.
     """
     tree_defs = {}
     for mod, files in mod_files_list:
@@ -127,7 +165,7 @@ def _repr_tree_defs(data, indent_str=None):
     return "\n".join(lines)
 
 
-def _dependencies_graph(filename: str, dep_info: Dict[str, Set[str]]) -> str:
+def _dependencies_graph(filename: str, dep_info: dict[str, set[str]]) -> str:
     """Write dependencies as a dot (graphviz) file."""
     done = {}
     printer = DotBackend(os.path.splitext(os.path.basename(filename))[0], rankdir="LR")
@@ -147,10 +185,10 @@ def _dependencies_graph(filename: str, dep_info: Dict[str, Set[str]]) -> str:
 
 
 def _make_graph(
-    filename: str, dep_info: Dict[str, Set[str]], sect: Section, gtype: str
+    filename: str, dep_info: dict[str, set[str]], sect: Section, gtype: str
 ):
     """Generate a dependencies graph and add some information about it in the
-    report's section
+    report's section.
     """
     outputfile = _dependencies_graph(filename, dep_info)
     sect.append(Paragraph((f"{gtype}imports graph has been written to {outputfile}",)))
@@ -158,7 +196,8 @@ def _make_graph(
 
 # the import checker itself ###################################################
 
-MSGS = {
+MSGS: dict[str, MessageDefinitionTuple] = {
+    **{k: v for k, v in DeprecatedMixin.msgs.items() if k[1:3] == "04"},
     "E0401": (
         "Unable to import %s",
         "import-error",
@@ -189,11 +228,6 @@ MSGS = {
         "wildcard-import",
         "Used when `from module import *` is detected.",
     ),
-    "W0402": (
-        "Uses of a deprecated module %r",
-        "deprecated-module",
-        "Used a module marked as deprecated is imported.",
-    ),
     "W0404": (
         "Reimport %r (imported line %s)",
         "reimported",
@@ -267,11 +301,8 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
     * uses of modules instead of preferred modules
     """
 
-    __implements__ = IAstroidChecker
-
     name = "imports"
     msgs = MSGS
-    priority = -2
     default_deprecated_modules = ()
 
     options = (
@@ -299,7 +330,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             "import-graph",
             {
                 "default": "",
-                "type": "string",
+                "type": "path",
                 "metavar": "<file.gv>",
                 "help": "Output a graph (.gv or any supported image format) of"
                 " all (i.e. internal and external) dependencies to the given file"
@@ -310,7 +341,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             "ext-import-graph",
             {
                 "default": "",
-                "type": "string",
+                "type": "path",
                 "metavar": "<file.gv>",
                 "help": "Output a graph (.gv or any supported image format)"
                 " of external dependencies to the given file"
@@ -321,7 +352,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             "int-import-graph",
             {
                 "default": "",
-                "type": "string",
+                "type": "path",
                 "metavar": "<file.gv>",
                 "help": "Output a graph (.gv or any supported image format)"
                 " of internal dependencies to the given file"
@@ -360,19 +391,6 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                 ),
             },
         ),
-        (
-            "analyse-fallback-blocks",
-            {
-                "default": False,
-                "type": "yn",
-                "metavar": "<y or n>",
-                "help": "Analyse import fallback blocks. This can be used to "
-                "support both Python 2 and 3 compatible code, which "
-                "means that the block might have code that exists "
-                "only in one or another interpreter, leading to false "
-                "positives when analysed.",
-            },
-        ),
         (
             "allow-wildcard-with-all",
             {
@@ -384,15 +402,15 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         ),
     )
 
-    def __init__(self, linter: Optional["PyLinter"] = None) -> None:
+    def __init__(self, linter: PyLinter) -> None:
         BaseChecker.__init__(self, linter)
         self.import_graph: collections.defaultdict = collections.defaultdict(set)
-        self._imports_stack: List[Tuple[Any, Any]] = []
+        self._imports_stack: list[tuple[Any, Any]] = []
         self._first_non_import_node = None
-        self._module_pkg: Dict[
+        self._module_pkg: dict[
             Any, Any
         ] = {}  # mapping of modules to the pkg they belong in
-        self._allow_any_import_level: Set[Any] = set()
+        self._allow_any_import_level: set[Any] = set()
         self.reports = (
             ("RP0401", "External dependencies", self._report_external_dependencies),
             ("RP0402", "Modules dependencies graph", self._report_dependencies_graph),
@@ -405,14 +423,14 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         self.import_graph = collections.defaultdict(set)
         self._module_pkg = {}  # mapping of modules to the pkg they belong in
         self._excluded_edges = collections.defaultdict(set)
-        self._ignored_modules = get_global_option(self, "ignored-modules", default=[])
+        self._ignored_modules = self.linter.config.ignored_modules
         # Build a mapping {'module': 'preferred-module'}
         self.preferred_modules = dict(
             module.split(":")
-            for module in self.config.preferred_modules
+            for module in self.linter.config.preferred_modules
             if ":" in module
         )
-        self._allow_any_import_level = set(self.config.allow_any_import_level)
+        self._allow_any_import_level = set(self.linter.config.allow_any_import_level)
 
     def _import_graph_without_ignored_edges(self):
         filtered_graph = copy.deepcopy(self.import_graph)
@@ -428,11 +446,16 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             for cycle in get_cycles(graph, vertices=vertices):
                 self.add_message("cyclic-import", args=" -> ".join(cycle))
 
-    def deprecated_modules(self):
+    def deprecated_modules(self) -> set[str]:
         """Callback returning the deprecated modules."""
-        return self.config.deprecated_modules
+        # First get the modules the user indicated
+        all_deprecated_modules = set(self.linter.config.deprecated_modules)
+        # Now get the hard-coded ones from the stdlib
+        for since_vers, mod_set in DEPRECATED_MODULES.items():
+            if since_vers <= sys.version_info:
+                all_deprecated_modules = all_deprecated_modules.union(mod_set)
+        return all_deprecated_modules
 
-    @check_messages(*MSGS)
     def visit_import(self, node: nodes.Import) -> None:
         """Triggered when an import statement is seen."""
         self._check_reimport(node)
@@ -458,15 +481,15 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
 
             self._add_imported_module(node, imported_module.name)
 
-    @check_messages(*MSGS)
     def visit_importfrom(self, node: nodes.ImportFrom) -> None:
         """Triggered when a from statement is seen."""
         basename = node.modname
         imported_module = self._get_imported_module(node, basename)
+        absolute_name = get_import_name(node, basename)
 
         self._check_import_as_rename(node)
         self._check_misplaced_future(node)
-        self.check_deprecated_module(node, basename)
+        self.check_deprecated_module(node, absolute_name)
         self._check_preferred_module(node, basename)
         self._check_wildcard_imports(node, imported_module)
         self._check_same_line_imports(node)
@@ -486,14 +509,13 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             else:
                 self._add_imported_module(node, imported_module.name)
 
-    @check_messages(*MSGS)
     def leave_module(self, node: nodes.Module) -> None:
         # Check imports are grouped by category (standard, 3rd party, local)
         std_imports, ext_imports, loc_imports = self._check_imports_order(node)
 
         # Check that imports are grouped by package within a given category
-        met_import: Set[str] = set()  # set for 'import x' style
-        met_from: Set[str] = set()  # set for 'from x import y' style
+        met_import: set[str] = set()  # set for 'import x' style
+        met_from: set[str] = set()  # set for 'from x import y' style
         current_package = None
         for import_node, import_name in std_imports + ext_imports + loc_imports:
             met = met_from if isinstance(import_node, nodes.ImportFrom) else met_import
@@ -516,8 +538,6 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         self._first_non_import_node = None
 
     def compute_first_non_import_node(self, node):
-        if not self.linter.is_message_enabled("wrong-import-position", node.fromlineno):
-            return
         # if the node does not contain an import instruction, and if it is the
         # first node of the module, keep a track of it (all the import positions
         # of the module will be compared to the position of this first
@@ -558,8 +578,6 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
     ) = visit_comprehension = visit_expr = visit_if = compute_first_non_import_node
 
     def visit_functiondef(self, node: nodes.FunctionDef) -> None:
-        if not self.linter.is_message_enabled("wrong-import-position", node.fromlineno):
-            return
         # If it is the first non import instruction of the module, record it.
         if self._first_non_import_node:
             return
@@ -610,7 +628,16 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         # if a first non-import instruction has already been encountered,
         # it means the import comes after it and therefore is not well placed
         if self._first_non_import_node:
-            self.add_message("wrong-import-position", node=node, args=node.as_string())
+            if self.linter.is_message_enabled(
+                "wrong-import-position", self._first_non_import_node.fromlineno
+            ):
+                self.add_message(
+                    "wrong-import-position", node=node, args=node.as_string()
+                )
+            else:
+                self.linter.add_ignored_message(
+                    "wrong-import-position", node.fromlineno, node
+                )
 
     def _record_import(self, node, importedmodnode):
         """Record the package `node` imports from."""
@@ -652,7 +679,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         third_party_not_ignored = []
         first_party_not_ignored = []
         local_not_ignored = []
-        isort_driver = IsortDriver(self.config)
+        isort_driver = IsortDriver(self.linter.config)
         for node, modname in self._imports_stack:
             if modname.startswith("."):
                 package = "." + modname.split(".")[1]
@@ -744,22 +771,25 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             message = f"Cannot import {modname!r} due to syntax error {str(exc.error)!r}"  # pylint: disable=no-member; false positive
             self.add_message("syntax-error", line=importnode.lineno, args=message)
 
-        except astroid.AstroidBuildingException:
+        except astroid.AstroidBuildingError:
             if not self.linter.is_message_enabled("import-error"):
                 return None
             if _ignore_import_failure(importnode, modname, self._ignored_modules):
                 return None
-            if not self.config.analyse_fallback_blocks and is_from_fallback_block(
-                importnode
+            if (
+                not self.linter.config.analyse_fallback_blocks
+                and is_from_fallback_block(importnode)
             ):
                 return None
 
             dotted_modname = get_import_name(importnode, modname)
             self.add_message("import-error", args=repr(dotted_modname), node=importnode)
+        except Exception as e:  # pragma: no cover
+            raise astroid.AstroidError from e
         return None
 
     def _add_imported_module(
-        self, node: Union[nodes.Import, nodes.ImportFrom], importedmodname: str
+        self, node: nodes.Import | nodes.ImportFrom, importedmodname: str
     ) -> None:
         """Notify an imported module, used to analyze dependencies."""
         module_file = node.root().file
@@ -788,7 +818,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                 self._module_pkg[context_name] = context_name.rsplit(".", 1)[0]
 
             # handle dependencies
-            dependencies_stat: Dict[str, Set[str]] = self.linter.stats.dependencies
+            dependencies_stat: dict[str, set[str]] = self.linter.stats.dependencies
             importedmodnames = dependencies_stat.setdefault(importedmodname, set())
             if context_name not in importedmodnames:
                 importedmodnames.add(context_name)
@@ -810,9 +840,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
                 args=(self.preferred_modules[mod_path], mod_path),
             )
 
-    def _check_import_as_rename(
-        self, node: Union[nodes.Import, nodes.ImportFrom]
-    ) -> None:
+    def _check_import_as_rename(self, node: nodes.Import | nodes.ImportFrom) -> None:
         names = node.names
         for name in names:
             if not all(name):
@@ -866,18 +894,18 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
         """Write dependencies as a dot (graphviz) file."""
         dep_info = self.linter.stats.dependencies
         if not dep_info or not (
-            self.config.import_graph
-            or self.config.ext_import_graph
-            or self.config.int_import_graph
+            self.linter.config.import_graph
+            or self.linter.config.ext_import_graph
+            or self.linter.config.int_import_graph
         ):
             raise EmptyReportError()
-        filename = self.config.import_graph
+        filename = self.linter.config.import_graph
         if filename:
             _make_graph(filename, dep_info, sect, "")
-        filename = self.config.ext_import_graph
+        filename = self.linter.config.ext_import_graph
         if filename:
             _make_graph(filename, self._external_dependencies_info(), sect, "external ")
-        filename = self.config.int_import_graph
+        filename = self.linter.config.int_import_graph
         if filename:
             _make_graph(filename, self._internal_dependencies_info(), sect, "internal ")
 
@@ -895,14 +923,14 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
     @astroid.decorators.cached
     def _external_dependencies_info(self):
         """Return cached external dependencies information or build and
-        cache them
+        cache them.
         """
         return self._filter_dependencies_graph(internal=False)
 
     @astroid.decorators.cached
     def _internal_dependencies_info(self):
         """Return cached internal dependencies information or build and
-        cache them
+        cache them.
         """
         return self._filter_dependencies_graph(internal=True)
 
@@ -918,7 +946,7 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
 
     def _wildcard_import_is_allowed(self, imported_module):
         return (
-            self.config.allow_wildcard_with_all
+            self.linter.config.allow_wildcard_with_all
             and imported_module is not None
             and "__all__" in imported_module.locals
         )
@@ -948,5 +976,5 @@ class ImportsChecker(DeprecatedMixin, BaseChecker):
             )
 
 
-def register(linter: "PyLinter") -> None:
+def register(linter: PyLinter) -> None:
     linter.register_checker(ImportsChecker(linter))
