@@ -3,34 +3,32 @@
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Some functions that may be useful for various checkers."""
+
+from __future__ import annotations
+
 import builtins
 import itertools
 import numbers
 import re
 import string
 import warnings
+from collections.abc import Iterable
 from functools import lru_cache, partial
-from typing import (
-    Callable,
-    Dict,
-    Iterable,
-    List,
-    Match,
-    Optional,
-    Set,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-)
+from re import Match
+from typing import TYPE_CHECKING, Callable, TypeVar
 
 import _string
-import astroid
 import astroid.objects
 from astroid import TooManyLevelsError, nodes
 from astroid.context import InferenceContext
+from astroid.exceptions import AstroidError
 
-from pylint.constants import TYPING_TYPE_CHECKS_GUARDS
+if TYPE_CHECKING:
+    from pylint.checkers import BaseChecker
+
+_NodeT = TypeVar("_NodeT", bound=nodes.NodeNG)
+_CheckerT = TypeVar("_CheckerT", bound="BaseChecker")
+AstCallbackMethod = Callable[[_CheckerT, _NodeT], None]
 
 COMP_NODE_TYPES = (
     nodes.ListComp,
@@ -70,7 +68,7 @@ KEYS_METHOD = "keys"
 #          although it's best to implement it in order to accept
 #          all of them.
 _SPECIAL_METHODS_PARAMS = {
-    None: ("__new__", "__init__", "__call__"),
+    None: ("__new__", "__init__", "__call__", "__init_subclass__"),
     0: (
         "__del__",
         "__repr__",
@@ -108,7 +106,6 @@ _SPECIAL_METHODS_PARAMS = {
         "__anext__",
         "__fspath__",
         "__subclasses__",
-        "__init_subclass__",
     ),
     1: (
         "__format__",
@@ -230,8 +227,6 @@ SUBSCRIPTABLE_CLASSES_PEP585 = frozenset(
     )
 )
 
-T_Node = TypeVar("T_Node", bound=nodes.NodeNG)
-
 
 class NoSuchArgumentError(Exception):
     pass
@@ -363,11 +358,9 @@ def is_defined_before(var_node: nodes.Name) -> bool:
     return False
 
 
-def is_default_argument(
-    node: nodes.NodeNG, scope: Optional[nodes.NodeNG] = None
-) -> bool:
+def is_default_argument(node: nodes.NodeNG, scope: nodes.NodeNG | None = None) -> bool:
     """Return true if the given Name node is used in function or lambda
-    default argument's value
+    default argument's value.
     """
     if not scope:
         scope = node.scope()
@@ -403,7 +396,7 @@ def is_func_decorator(node: nodes.NodeNG) -> bool:
 
 def is_ancestor_name(frame: nodes.ClassDef, node: nodes.NodeNG) -> bool:
     """Return whether `frame` is an astroid.Class node with `node` in the
-    subtree of its bases attribute
+    subtree of its bases attribute.
     """
     if not isinstance(frame, nodes.ClassDef):
         return False
@@ -424,7 +417,7 @@ def assign_parent(node: nodes.NodeNG) -> nodes.NodeNG:
 
 def overrides_a_method(class_node: nodes.ClassDef, name: str) -> bool:
     """Return True if <name> is a method overridden from an ancestor
-    which is not the base object class
+    which is not the base object class.
     """
     for ancestor in class_node.ancestors():
         if ancestor.name == "object":
@@ -434,16 +427,48 @@ def overrides_a_method(class_node: nodes.ClassDef, name: str) -> bool:
     return False
 
 
-def check_messages(*messages: str) -> Callable:
-    """Decorator to store messages that are handled by a checker method."""
+def only_required_for_messages(
+    *messages: str,
+) -> Callable[
+    [AstCallbackMethod[_CheckerT, _NodeT]], AstCallbackMethod[_CheckerT, _NodeT]
+]:
+    """Decorator to store messages that are handled by a checker method as an
+    attribute of the function object.
 
-    def store_messages(func):
-        func.checks_msgs = messages
+    This information is used by ``ASTWalker`` to decide whether to call the decorated
+    method or not. If none of the messages is enabled, the method will be skipped.
+    Therefore, the list of messages must be well maintained at all times!
+    This decorator only has an effect on ``visit_*`` and ``leave_*`` methods
+    of a class inheriting from ``BaseChecker``.
+    """
+
+    def store_messages(
+        func: AstCallbackMethod[_CheckerT, _NodeT]
+    ) -> AstCallbackMethod[_CheckerT, _NodeT]:
+        setattr(func, "checks_msgs", messages)
         return func
 
     return store_messages
 
 
+def check_messages(
+    *messages: str,
+) -> Callable[
+    [AstCallbackMethod[_CheckerT, _NodeT]], AstCallbackMethod[_CheckerT, _NodeT]
+]:
+    """Kept for backwards compatibility, deprecated.
+
+    Use only_required_for_messages instead, which conveys the intent of the decorator much clearer.
+    """
+    warnings.warn(
+        "utils.check_messages will be removed in favour of calling "
+        "utils.only_required_for_messages in pylint 3.0",
+        DeprecationWarning,
+    )
+
+    return only_required_for_messages(*messages)
+
+
 class IncompleteFormatString(Exception):
     """A format string ended in the middle of a format specifier."""
 
@@ -453,14 +478,14 @@ class UnsupportedFormatCharacter(Exception):
     format characters.
     """
 
-    def __init__(self, index):
+    def __init__(self, index: int) -> None:
         super().__init__(index)
         self.index = index
 
 
 def parse_format_string(
     format_string: str,
-) -> Tuple[Set[str], int, Dict[str, str], List[str]]:
+) -> tuple[set[str], int, dict[str, str], list[str]]:
     """Parses a format string, returning a tuple (keys, num_args).
 
     Where 'keys' is the set of mapping keys in the format string, and 'num_args' is the number
@@ -472,7 +497,7 @@ def parse_format_string(
     pos_types = []
     num_args = 0
 
-    def next_char(i):
+    def next_char(i: int) -> tuple[int, str]:
         i += 1
         if i == len(format_string):
             raise IncompleteFormatString
@@ -534,14 +559,16 @@ def parse_format_string(
     return keys, num_args, key_types, pos_types
 
 
-def split_format_field_names(format_string) -> Tuple[str, Iterable[Tuple[bool, str]]]:
+def split_format_field_names(
+    format_string: str,
+) -> tuple[str, Iterable[tuple[bool, str]]]:
     try:
         return _string.formatter_field_name_split(format_string)
     except ValueError as e:
         raise IncompleteFormatString() from e
 
 
-def collect_string_fields(format_string) -> Iterable[Optional[str]]:
+def collect_string_fields(format_string: str) -> Iterable[str | None]:
     """Given a format string, return an iterator
     of all the valid format fields.
 
@@ -576,7 +603,7 @@ def collect_string_fields(format_string) -> Iterable[Optional[str]]:
 
 def parse_format_method_string(
     format_string: str,
-) -> Tuple[List[Tuple[str, List[Tuple[bool, str]]]], int, int]:
+) -> tuple[list[tuple[str, list[tuple[bool, str]]]], int, int]:
     """Parses a PEP 3101 format string, returning a tuple of
     (keyword_arguments, implicit_pos_args_cnt, explicit_pos_args).
 
@@ -614,7 +641,7 @@ def is_attr_protected(attrname: str) -> bool:
     )
 
 
-def node_frame_class(node: nodes.NodeNG) -> Optional[nodes.ClassDef]:
+def node_frame_class(node: nodes.NodeNG) -> nodes.ClassDef | None:
     """Return the class that is wrapping the given node.
 
     The function returns a class for a method node (or a staticmethod or a
@@ -639,23 +666,23 @@ def node_frame_class(node: nodes.NodeNG) -> Optional[nodes.ClassDef]:
     return klass
 
 
-def get_outer_class(class_node: astroid.ClassDef) -> Optional[astroid.ClassDef]:
+def get_outer_class(class_node: astroid.ClassDef) -> astroid.ClassDef | None:
     """Return the class that is the outer class of given (nested) class_node."""
     parent_klass = class_node.parent.frame(future=True)
 
     return parent_klass if isinstance(parent_klass, astroid.ClassDef) else None
 
 
-def is_attr_private(attrname: str) -> Optional[Match[str]]:
+def is_attr_private(attrname: str) -> Match[str] | None:
     """Check that attribute name is private (at least two leading underscores,
-    at most one trailing underscore)
+    at most one trailing underscore).
     """
     regex = re.compile("^_{2,}.*[^_]+_?$")
     return regex.match(attrname)
 
 
 def get_argument_from_call(
-    call_node: nodes.Call, position: Optional[int] = None, keyword: Optional[str] = None
+    call_node: nodes.Call, position: int | None = None, keyword: str | None = None
 ) -> nodes.Name:
     """Returns the specified argument from a function call.
 
@@ -684,7 +711,7 @@ def get_argument_from_call(
     raise NoSuchArgumentError
 
 
-def inherit_from_std_ex(node: nodes.NodeNG) -> bool:
+def inherit_from_std_ex(node: nodes.NodeNG | astroid.Instance) -> bool:
     """Return whether the given class node is subclass of
     exceptions.Exception.
     """
@@ -696,7 +723,10 @@ def inherit_from_std_ex(node: nodes.NodeNG) -> bool:
     )
 
 
-def error_of_type(handler: nodes.ExceptHandler, error_type) -> bool:
+def error_of_type(
+    handler: nodes.ExceptHandler,
+    error_type: str | type[Exception] | tuple[str | type[Exception], ...],
+) -> bool:
     """Check if the given exception handler catches
     the given error_type.
 
@@ -707,7 +737,7 @@ def error_of_type(handler: nodes.ExceptHandler, error_type) -> bool:
     given errors.
     """
 
-    def stringify_error(error):
+    def stringify_error(error: str | type[Exception]) -> str:
         if not isinstance(error, str):
             return error.__name__
         return error
@@ -733,7 +763,7 @@ def decorated_with_property(node: nodes.FunctionDef) -> bool:
     return False
 
 
-def _is_property_kind(node, *kinds):
+def _is_property_kind(node, *kinds: str) -> bool:
     if not isinstance(node, (astroid.UnboundMethod, nodes.FunctionDef)):
         return False
     if node.decorators:
@@ -743,17 +773,17 @@ def _is_property_kind(node, *kinds):
     return False
 
 
-def is_property_setter(node: nodes.FunctionDef) -> bool:
+def is_property_setter(node) -> bool:
     """Check if the given node is a property setter."""
     return _is_property_kind(node, "setter")
 
 
-def is_property_deleter(node: nodes.FunctionDef) -> bool:
+def is_property_deleter(node) -> bool:
     """Check if the given node is a property deleter."""
     return _is_property_kind(node, "deleter")
 
 
-def is_property_setter_or_deleter(node: nodes.FunctionDef) -> bool:
+def is_property_setter_or_deleter(node) -> bool:
     """Check if the given node is either a property setter or a deleter."""
     return _is_property_kind(node, "setter", "deleter")
 
@@ -769,7 +799,7 @@ def _is_property_decorator(decorator: nodes.Name) -> bool:
         elif isinstance(inferred, nodes.FunctionDef):
             # If decorator is function, check if it has exactly one return
             # and the return is itself a function decorated with property
-            returns: List[nodes.Return] = list(
+            returns: list[nodes.Return] = list(
                 inferred._get_return_nodes_skip_functions()
             )
             if len(returns) == 1 and isinstance(
@@ -786,9 +816,9 @@ def _is_property_decorator(decorator: nodes.Name) -> bool:
 
 
 def decorated_with(
-    func: Union[
-        nodes.ClassDef, nodes.FunctionDef, astroid.BoundMethod, astroid.UnboundMethod
-    ],
+    func: (
+        nodes.ClassDef | nodes.FunctionDef | astroid.BoundMethod | astroid.UnboundMethod
+    ),
     qnames: Iterable[str],
 ) -> bool:
     """Determine if the `func` node has a decorator with the qualified name `qname`."""
@@ -811,7 +841,7 @@ def decorated_with(
 
 def uninferable_final_decorators(
     node: nodes.Decorators,
-) -> List[Optional[Union[nodes.Attribute, nodes.Name]]]:
+) -> list[nodes.Attribute | nodes.Name | None]:
     """Return a list of uninferable `typing.final` decorators in `node`.
 
     This function is used to determine if the `typing.final` decorator is used
@@ -820,28 +850,34 @@ def uninferable_final_decorators(
     """
     decorators = []
     for decorator in getattr(node, "nodes", []):
+        import_nodes: tuple[nodes.Import | nodes.ImportFrom] | None = None
+
+        # Get the `Import` node. The decorator is of the form: @module.name
         if isinstance(decorator, nodes.Attribute):
-            try:
-                import_node = decorator.expr.lookup(decorator.expr.name)[1][0]
-            except AttributeError:
-                continue
+            inferred = safe_infer(decorator.expr)
+            if isinstance(inferred, nodes.Module) and inferred.qname() == "typing":
+                _, import_nodes = decorator.expr.lookup(decorator.expr.name)
+
+        # Get the `ImportFrom` node. The decorator is of the form: @name
         elif isinstance(decorator, nodes.Name):
-            lookup_values = decorator.lookup(decorator.name)
-            if lookup_values[1]:
-                import_node = lookup_values[1][0]
-            else:
-                continue  # pragma: no cover # Covered on Python < 3.8
-        else:
+            _, import_nodes = decorator.lookup(decorator.name)
+
+        # The `final` decorator is expected to be found in the
+        # import_nodes. Continue if we don't find any `Import` or `ImportFrom`
+        # nodes for this decorator.
+        if not import_nodes:
             continue
+        import_node = import_nodes[0]
 
         if not isinstance(import_node, (astroid.Import, astroid.ImportFrom)):
             continue
 
         import_names = dict(import_node.names)
 
-        # from typing import final
+        # Check if the import is of the form: `from typing import final`
         is_from_import = ("final" in import_names) and import_node.modname == "typing"
-        # import typing
+
+        # Check if the import is of the form: `import typing`
         is_import = ("typing" in import_names) and getattr(
             decorator, "attrname", None
         ) == "final"
@@ -857,7 +893,7 @@ def uninferable_final_decorators(
 @lru_cache(maxsize=1024)
 def unimplemented_abstract_methods(
     node: nodes.ClassDef, is_abstract_cb: nodes.FunctionDef = None
-) -> Dict[str, nodes.NodeNG]:
+) -> dict[str, nodes.NodeNG]:
     """Get the unimplemented abstract methods for the given *node*.
 
     A method can be considered abstract if the callback *is_abstract_cb*
@@ -870,7 +906,7 @@ def unimplemented_abstract_methods(
     """
     if is_abstract_cb is None:
         is_abstract_cb = partial(decorated_with, qnames=ABC_METHODS)
-    visited: Dict[str, nodes.NodeNG] = {}
+    visited: dict[str, nodes.NodeNG] = {}
     try:
         mro = reversed(node.mro())
     except NotImplementedError:
@@ -914,7 +950,7 @@ def unimplemented_abstract_methods(
 
 def find_try_except_wrapper_node(
     node: nodes.NodeNG,
-) -> Optional[Union[nodes.ExceptHandler, nodes.TryExcept]]:
+) -> nodes.ExceptHandler | nodes.TryExcept | None:
     """Return the ExceptHandler or the TryExcept node in which the node is."""
     current = node
     ignores = (nodes.ExceptHandler, nodes.TryExcept)
@@ -928,7 +964,7 @@ def find_try_except_wrapper_node(
 
 def find_except_wrapper_node_in_scope(
     node: nodes.NodeNG,
-) -> Optional[Union[nodes.ExceptHandler, nodes.TryExcept]]:
+) -> nodes.ExceptHandler | nodes.TryExcept | None:
     """Return the ExceptHandler in which the node is, without going out of scope."""
     for current in node.node_ancestors():
         if isinstance(current, astroid.scoped_nodes.LocalsDictNodeNG):
@@ -969,15 +1005,15 @@ def is_from_fallback_block(node: nodes.NodeNG) -> bool:
 
 def _except_handlers_ignores_exceptions(
     handlers: nodes.ExceptHandler,
-    exceptions: Tuple[Type[ImportError], Type[ModuleNotFoundError]],
+    exceptions: tuple[type[ImportError], type[ModuleNotFoundError]],
 ) -> bool:
     func = partial(error_of_type, error_type=exceptions)
     return any(func(handler) for handler in handlers)
 
 
 def get_exception_handlers(
-    node: nodes.NodeNG, exception=Exception
-) -> Optional[List[nodes.ExceptHandler]]:
+    node: nodes.NodeNG, exception: type[Exception] = Exception
+) -> list[nodes.ExceptHandler] | None:
     """Return the collections of handlers handling the exception in arguments.
 
     Args:
@@ -986,7 +1022,6 @@ def get_exception_handlers(
 
     Returns:
         list: the collection of handlers that are handling the exception or None.
-
     """
     context = find_try_except_wrapper_node(node)
     if isinstance(context, nodes.TryExcept):
@@ -1010,7 +1045,9 @@ def is_node_inside_try_except(node: nodes.Raise) -> bool:
     return isinstance(context, nodes.TryExcept)
 
 
-def node_ignores_exception(node: nodes.NodeNG, exception=Exception) -> bool:
+def node_ignores_exception(
+    node: nodes.NodeNG, exception: type[Exception] = Exception
+) -> bool:
     """Check if the node is in a TryExcept which handles the given exception.
 
     If the exception is not given, the function is going to look for bare
@@ -1024,7 +1061,7 @@ def node_ignores_exception(node: nodes.NodeNG, exception=Exception) -> bool:
 
 def class_is_abstract(node: nodes.ClassDef) -> bool:
     """Return true if the given class node should be considered as an abstract
-    class
+    class.
     """
     # Only check for explicit metaclass=ABCMeta on this specific class
     meta = node.declared_metaclass()
@@ -1131,7 +1168,7 @@ def is_inside_abstract_class(node: nodes.NodeNG) -> bool:
 
 
 def _supports_protocol(
-    value: nodes.NodeNG, protocol_callback: nodes.FunctionDef
+    value: nodes.NodeNG, protocol_callback: Callable[[nodes.NodeNG], bool]
 ) -> bool:
     if isinstance(value, nodes.ClassDef):
         if not has_known_bases(value):
@@ -1149,8 +1186,6 @@ def _supports_protocol(
         if protocol_callback(value):
             return True
 
-    # pylint: disable-next=fixme
-    # TODO: Should be covered by https://github.com/PyCQA/astroid/pull/1475
     if isinstance(value, nodes.ComprehensionScope):
         return True
 
@@ -1186,7 +1221,9 @@ def supports_getitem(value: nodes.NodeNG, node: nodes.NodeNG) -> bool:
     if isinstance(value, nodes.ClassDef):
         if _supports_protocol_method(value, CLASS_GETITEM_METHOD):
             return True
-        if is_class_subscriptable_pep585_with_postponed_evaluation_enabled(value, node):
+        if is_postponed_evaluation_enabled(node) and is_node_in_type_annotation_context(
+            node
+        ):
             return True
     return _supports_protocol(value, _supports_getitem_protocol)
 
@@ -1199,8 +1236,8 @@ def supports_delitem(value: nodes.NodeNG, _: nodes.NodeNG) -> bool:
     return _supports_protocol(value, _supports_delitem_protocol)
 
 
-def _get_python_type_of_node(node: nodes.NodeNG) -> Optional[str]:
-    pytype = getattr(node, "pytype", None)
+def _get_python_type_of_node(node: nodes.NodeNG) -> str | None:
+    pytype: Callable[[], str] | None = getattr(node, "pytype", None)
     if callable(pytype):
         return pytype()
     return None
@@ -1208,19 +1245,21 @@ def _get_python_type_of_node(node: nodes.NodeNG) -> Optional[str]:
 
 @lru_cache(maxsize=1024)
 def safe_infer(
-    node: nodes.NodeNG, context: Optional[InferenceContext] = None
-) -> Union[nodes.NodeNG, Type[astroid.Uninferable], None]:
+    node: nodes.NodeNG, context: InferenceContext | None = None
+) -> nodes.NodeNG | type[astroid.Uninferable] | None:
     """Return the inferred value for the given node.
 
     Return None if inference failed or if there is some ambiguity (more than
     one node has been inferred of different types).
     """
-    inferred_types: Set[Optional[str]] = set()
+    inferred_types: set[str | None] = set()
     try:
         infer_gen = node.infer(context=context)
         value = next(infer_gen)
     except astroid.InferenceError:
         return None
+    except Exception as e:  # pragma: no cover
+        raise AstroidError from e
 
     if value is not astroid.Uninferable:
         inferred_types.add(_get_python_type_of_node(value))
@@ -1242,20 +1281,26 @@ def safe_infer(
         return None  # There is some kind of ambiguity
     except StopIteration:
         return value
+    except Exception as e:  # pragma: no cover
+        raise AstroidError from e
     return value if len(inferred_types) <= 1 else None
 
 
 @lru_cache(maxsize=512)
 def infer_all(
     node: nodes.NodeNG, context: InferenceContext = None
-) -> List[nodes.NodeNG]:
+) -> list[nodes.NodeNG]:
     try:
         return list(node.infer(context=context))
     except astroid.InferenceError:
         return []
+    except Exception as e:  # pragma: no cover
+        raise AstroidError from e
 
 
-def has_known_bases(klass: nodes.ClassDef, context=None) -> bool:
+def has_known_bases(
+    klass: nodes.ClassDef, context: InferenceContext | None = None
+) -> bool:
     """Return true if all base classes of a class could be inferred."""
     try:
         return klass._all_bases_known
@@ -1282,7 +1327,7 @@ def is_none(node: nodes.NodeNG) -> bool:
     )
 
 
-def node_type(node: nodes.NodeNG) -> Optional[nodes.NodeNG]:
+def node_type(node: nodes.NodeNG) -> nodes.NodeNG | None:
     """Return the inferred type for `node`.
 
     If there is more than one possible type, or if inferred type is Uninferable or None,
@@ -1290,7 +1335,7 @@ def node_type(node: nodes.NodeNG) -> Optional[nodes.NodeNG]:
     """
     # check there is only one possible type for the assign node. Else we
     # don't handle it for now
-    types: Set[nodes.NodeNG] = set()
+    types: set[nodes.NodeNG] = set()
     try:
         for var_type in node.infer():
             if var_type == astroid.Uninferable or is_none(var_type):
@@ -1371,6 +1416,13 @@ def is_class_subscriptable_pep585_with_postponed_evaluation_enabled(
     """Check if class is subscriptable with PEP 585 and
     postponed evaluation enabled.
     """
+    warnings.warn(
+        "'is_class_subscriptable_pep585_with_postponed_evaluation_enabled' has been "
+        "deprecated and will be removed in pylint 3.0. "
+        "Use 'is_postponed_evaluation_enabled(node) and "
+        "is_node_in_type_annotation_context(node)' instead.",
+        DeprecationWarning,
+    )
     return (
         is_postponed_evaluation_enabled(node)
         and value.qname() in SUBSCRIPTABLE_CLASSES_PEP585
@@ -1382,7 +1434,7 @@ def is_node_in_type_annotation_context(node: nodes.NodeNG) -> bool:
     """Check if node is in type annotation context.
 
     Check for 'AnnAssign', function 'Arguments',
-    or part of function return type anntation.
+    or part of function return type annotation.
     """
     # pylint: disable=too-many-boolean-expressions
     current_node, parent_node = node, node.parent
@@ -1463,7 +1515,7 @@ def is_call_of_name(node: nodes.NodeNG, name: str) -> bool:
 
 def is_test_condition(
     node: nodes.NodeNG,
-    parent: Optional[nodes.NodeNG] = None,
+    parent: nodes.NodeNG | None = None,
 ) -> bool:
     """Returns true if the given node is being tested for truthiness."""
     parent = parent or node.parent
@@ -1482,7 +1534,7 @@ def is_classdef_type(node: nodes.ClassDef) -> bool:
 
 
 def is_attribute_typed_annotation(
-    node: Union[nodes.ClassDef, astroid.Instance], attr_name: str
+    node: nodes.ClassDef | astroid.Instance, attr_name: str
 ) -> bool:
     """Test if attribute is typed annotation in current node
     or any base nodes.
@@ -1526,9 +1578,7 @@ def is_assign_name_annotated_with(node: nodes.AssignName, typing_name: str) -> b
     return False
 
 
-def get_iterating_dictionary_name(
-    node: Union[nodes.For, nodes.Comprehension]
-) -> Optional[str]:
+def get_iterating_dictionary_name(node: nodes.For | nodes.Comprehension) -> str | None:
     """Get the name of the dictionary which keys are being iterated over on
     a ``nodes.For`` or ``nodes.Comprehension`` node.
 
@@ -1570,9 +1620,7 @@ def get_subscript_const_value(node: nodes.Subscript) -> nodes.Const:
     return inferred
 
 
-def get_import_name(
-    importnode: Union[nodes.Import, nodes.ImportFrom], modname: str
-) -> str:
+def get_import_name(importnode: nodes.Import | nodes.ImportFrom, modname: str) -> str:
     """Get a prepared module name from the given import node.
 
     In the case of relative imports, this will return the
@@ -1645,7 +1693,9 @@ def is_node_in_guarded_import_block(node: nodes.NodeNG) -> bool:
 
 
 def is_reassigned_after_current(node: nodes.NodeNG, varname: str) -> bool:
-    """Check if the given variable name is reassigned in the same scope after the current node."""
+    """Check if the given variable name is reassigned in the same scope after the
+    current node.
+    """
     return any(
         a.name == varname and a.lineno > node.lineno
         for a in node.scope().nodes_of_class(
@@ -1655,7 +1705,9 @@ def is_reassigned_after_current(node: nodes.NodeNG, varname: str) -> bool:
 
 
 def is_deleted_after_current(node: nodes.NodeNG, varname: str) -> bool:
-    """Check if the given variable name is deleted in the same scope after the current node."""
+    """Check if the given variable name is deleted in the same scope after the current
+    node.
+    """
     return any(
         getattr(target, "name", None) == varname and target.lineno > node.lineno
         for del_node in node.scope().nodes_of_class(nodes.Delete)
@@ -1673,15 +1725,15 @@ def is_function_body_ellipsis(node: nodes.FunctionDef) -> bool:
     )
 
 
-def is_base_container(node: Optional[nodes.NodeNG]) -> bool:
+def is_base_container(node: nodes.NodeNG | None) -> bool:
     return isinstance(node, nodes.BaseContainer) and not node.elts
 
 
-def is_empty_dict_literal(node: Optional[nodes.NodeNG]) -> bool:
+def is_empty_dict_literal(node: nodes.NodeNG | None) -> bool:
     return isinstance(node, nodes.Dict) and not node.items
 
 
-def is_empty_str_literal(node: Optional[nodes.NodeNG]) -> bool:
+def is_empty_str_literal(node: nodes.NodeNG | None) -> bool:
     return (
         isinstance(node, nodes.Const) and isinstance(node.value, str) and not node.value
     )
@@ -1697,8 +1749,8 @@ def returns_bool(node: nodes.NodeNG) -> bool:
 
 
 def get_node_first_ancestor_of_type(
-    node: nodes.NodeNG, ancestor_type: Union[Type[T_Node], Tuple[Type[T_Node], ...]]
-) -> Optional[T_Node]:
+    node: nodes.NodeNG, ancestor_type: type[_NodeT] | tuple[type[_NodeT], ...]
+) -> _NodeT | None:
     """Return the first parent node that is any of the provided types (or None)."""
     for ancestor in node.node_ancestors():
         if isinstance(ancestor, ancestor_type):
@@ -1707,10 +1759,10 @@ def get_node_first_ancestor_of_type(
 
 
 def get_node_first_ancestor_of_type_and_its_child(
-    node: nodes.NodeNG, ancestor_type: Union[Type[T_Node], Tuple[Type[T_Node], ...]]
-) -> Union[Tuple[None, None], Tuple[T_Node, nodes.NodeNG]]:
+    node: nodes.NodeNG, ancestor_type: type[_NodeT] | tuple[type[_NodeT], ...]
+) -> tuple[None, None] | tuple[_NodeT, nodes.NodeNG]:
     """Modified version of get_node_first_ancestor_of_type to also return the
-    descendant visited directly before reaching the sought ancestor
+    descendant visited directly before reaching the sought ancestor.
 
     Useful for extracting whether a statement is guarded by a try, except, or finally
     when searching for a TryFinally ancestor.
@@ -1724,9 +1776,39 @@ def get_node_first_ancestor_of_type_and_its_child(
 
 
 def in_type_checking_block(node: nodes.NodeNG) -> bool:
-    """Check if a node is guarded by a TYPE_CHECKS guard."""
-    return any(
-        isinstance(ancestor, nodes.If)
-        and ancestor.test.as_string() in TYPING_TYPE_CHECKS_GUARDS
-        for ancestor in node.node_ancestors()
+    """Check if a node is guarded by a TYPE_CHECKING guard."""
+    for ancestor in node.node_ancestors():
+        if not isinstance(ancestor, nodes.If):
+            continue
+        if isinstance(ancestor.test, nodes.Name):
+            if ancestor.test.name != "TYPE_CHECKING":
+                continue
+            maybe_import_from = ancestor.test.lookup(ancestor.test.name)[1][0]
+            if (
+                isinstance(maybe_import_from, nodes.ImportFrom)
+                and maybe_import_from.modname == "typing"
+            ):
+                return True
+            inferred = safe_infer(ancestor.test)
+            if isinstance(inferred, nodes.Const) and inferred.value is False:
+                return True
+        elif isinstance(ancestor.test, nodes.Attribute):
+            if ancestor.test.attrname != "TYPE_CHECKING":
+                continue
+            inferred_module = safe_infer(ancestor.test.expr)
+            if (
+                isinstance(inferred_module, nodes.Module)
+                and inferred_module.name == "typing"
+                and inferred_module.pytype() == "builtins.module"
+            ):
+                return True
+
+    return False
+
+
+@lru_cache()
+def in_for_else_branch(parent: nodes.NodeNG, stmt: nodes.Statement) -> bool:
+    """Returns True if stmt is inside the else branch for a parent For stmt."""
+    return isinstance(parent, nodes.For) and any(
+        else_stmt.parent_of(stmt) or else_stmt == stmt for else_stmt in parent.orelse
     )
