@@ -4,6 +4,8 @@
 
 """Module to add McCabe checker class for pylint."""
 
+from __future__ import annotations
+
 from typing import TYPE_CHECKING
 
 from astroid import nodes
@@ -11,8 +13,8 @@ from mccabe import PathGraph as Mccabe_PathGraph
 from mccabe import PathGraphingAstVisitor as Mccabe_PathGraphingAstVisitor
 
 from pylint import checkers
-from pylint.checkers.utils import check_messages
-from pylint.interfaces import HIGH, IAstroidChecker
+from pylint.checkers.utils import only_required_for_messages
+from pylint.interfaces import HIGH
 
 if TYPE_CHECKING:
     from pylint.lint import PyLinter
@@ -146,7 +148,6 @@ class McCabeMethodChecker(checkers.BaseChecker):
     to validate a too complex code.
     """
 
-    __implements__ = IAstroidChecker
     name = "design"
 
     msgs = {
@@ -169,10 +170,10 @@ class McCabeMethodChecker(checkers.BaseChecker):
         ),
     )
 
-    @check_messages("too-complex")
+    @only_required_for_messages("too-complex")
     def visit_module(self, node: nodes.Module) -> None:
         """Visit an astroid.Module node to check too complex rating and
-        add message if is greater than max_complexity stored from options
+        add message if is greater than max_complexity stored from options.
         """
         visitor = PathGraphingAstVisitor()
         for child in node.body:
@@ -184,12 +185,12 @@ class McCabeMethodChecker(checkers.BaseChecker):
                 node_name = f"'{node.name}'"
             else:
                 node_name = f"This '{node.__class__.__name__.lower()}'"
-            if complexity <= self.config.max_complexity:
+            if complexity <= self.linter.config.max_complexity:
                 continue
             self.add_message(
                 "too-complex", node=node, confidence=HIGH, args=(node_name, complexity)
             )
 
 
-def register(linter: "PyLinter") -> None:
+def register(linter: PyLinter) -> None:
     linter.register_checker(McCabeMethodChecker(linter))
