@@ -2,9 +2,12 @@
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
+from __future__ import annotations
+
 import contextlib
 import sys
 import traceback
+from collections.abc import Iterator, Sequence
 from datetime import datetime
 from pathlib import Path
 
@@ -12,10 +15,6 @@ from pylint.config import PYLINT_HOME
 from pylint.lint.expand_modules import get_python_path
 
 
-class ArgumentPreprocessingError(Exception):
-    """Raised if an error occurs during argument preprocessing."""
-
-
 def prepare_crash_report(ex: Exception, filepath: str, crash_file_path: str) -> Path:
     issue_template_path = (
         Path(PYLINT_HOME) / datetime.now().strftime(str(crash_file_path))
@@ -72,42 +71,7 @@ def get_fatal_error_message(filepath: str, issue_template_path: Path) -> str:
     )
 
 
-def preprocess_options(args, search_for):
-    """Look for some options (keys of <search_for>) which have to be processed
-    before others
-
-    values of <search_for> are callback functions to call when the option is
-    found
-    """
-    i = 0
-    while i < len(args):
-        arg = args[i]
-        if not arg.startswith("--"):
-            i += 1
-        else:
-            try:
-                option, val = arg[2:].split("=", 1)
-            except ValueError:
-                option, val = arg[2:], None
-            try:
-                cb, takearg = search_for[option]
-            except KeyError:
-                i += 1
-            else:
-                del args[i]
-                if takearg and val is None:
-                    if i >= len(args) or args[i].startswith("-"):
-                        msg = f"Option {option} expects a value"
-                        raise ArgumentPreprocessingError(msg)
-                    val = args[i]
-                    del args[i]
-                elif not takearg and val is not None:
-                    msg = f"Option {option} doesn't expects a value"
-                    raise ArgumentPreprocessingError(msg)
-                cb(option, val)
-
-
-def _patch_sys_path(args):
+def _patch_sys_path(args: Sequence[str]) -> list[str]:
     original = list(sys.path)
     changes = []
     seen = set()
@@ -122,7 +86,7 @@ def _patch_sys_path(args):
 
 
 @contextlib.contextmanager
-def fix_import_path(args):
+def fix_import_path(args: Sequence[str]) -> Iterator[None]:
     """Prepare 'sys.path' for running the linter checks.
 
     Within this context, each of the given arguments is importable.
