@@ -3,20 +3,33 @@
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Generic classes/functions for pyreverse core/extensions."""
+
+from __future__ import annotations
+
 import os
 import re
 import shutil
 import subprocess
 import sys
-from typing import Optional, Union
+from typing import TYPE_CHECKING, Any, Callable, Optional, Tuple, Union
 
 import astroid
 from astroid import nodes
 
+if TYPE_CHECKING:
+    from pylint.pyreverse.diagrams import ClassDiagram, PackageDiagram
+
+    _CallbackT = Callable[
+        [nodes.NodeNG],
+        Union[Tuple[ClassDiagram], Tuple[PackageDiagram, ClassDiagram], None],
+    ]
+    _CallbackTupleT = Tuple[Optional[_CallbackT], Optional[_CallbackT]]
+
+
 RCFILE = ".pyreverserc"
 
 
-def get_default_options():
+def get_default_options() -> list[str]:
     """Read config file and return list of options."""
     options = []
     home = os.environ.get("HOME", "")
@@ -30,7 +43,7 @@ def get_default_options():
     return options
 
 
-def insert_default_options():
+def insert_default_options() -> None:
     """Insert default options to sys.argv."""
     options = get_default_options()
     options.reverse()
@@ -44,7 +57,7 @@ PRIVATE = re.compile(r"^__(_*[^\W_])+_?$")
 PROTECTED = re.compile(r"^_\w*$")
 
 
-def get_visibility(name):
+def get_visibility(name: str) -> str:
     """Return the visibility from a name: public, protected, private or special."""
     if SPECIAL.match(name):
         visibility = "special"
@@ -58,37 +71,18 @@ def get_visibility(name):
     return visibility
 
 
-ABSTRACT = re.compile(r"^.*Abstract.*")
-FINAL = re.compile(r"^[^\W\da-z]*$")
-
-
-def is_abstract(node):
-    """Return true if the given class node correspond to an abstract class
-    definition
-    """
-    return ABSTRACT.match(node.name)
-
-
-def is_final(node):
-    """Return true if the given class/function node correspond to final
-    definition
-    """
-    return FINAL.match(node.name)
-
-
-def is_interface(node):
-    # bw compat
+def is_interface(node: nodes.ClassDef) -> bool:
+    # bw compatibility
     return node.type == "interface"
 
 
-def is_exception(node):
-    # bw compat
+def is_exception(node: nodes.ClassDef) -> bool:
+    # bw compatibility
     return node.type == "exception"
 
 
 # Helpers #####################################################################
 
-_CONSTRUCTOR = 1
 _SPECIAL = 2
 _PROTECTED = 4
 _PRIVATE = 8
@@ -109,7 +103,7 @@ VIS_MOD = {
 class FilterMixIn:
     """Filter nodes according to a mode and nodes' visibility."""
 
-    def __init__(self, mode):
+    def __init__(self, mode: str) -> None:
         """Init filter modes."""
         __mode = 0
         for nummod in mode.split("+"):
@@ -119,14 +113,14 @@ class FilterMixIn:
                 print(f"Unknown filter mode {ex}", file=sys.stderr)
         self.__mode = __mode
 
-    def show_attr(self, node):
+    def show_attr(self, node: nodes.NodeNG | str) -> bool:
         """Return true if the node should be treated."""
         visibility = get_visibility(getattr(node, "name", node))
         return not self.__mode & VIS_MOD[visibility]
 
 
-class ASTWalker:
-    """A walker visiting a tree in preorder, calling on the handler:.
+class LocalsVisitor:
+    """Visit a project by traversing the locals dictionary.
 
     * visit_<class name> on entering a node, where class name is the class of
     the node in lower case
@@ -135,63 +129,28 @@ class ASTWalker:
     the node in lower case
     """
 
-    def __init__(self, handler):
-        self.handler = handler
-        self._cache = {}
+    def __init__(self) -> None:
+        self._cache: dict[type[nodes.NodeNG], _CallbackTupleT] = {}
+        self._visited: set[nodes.NodeNG] = set()
 
-    def walk(self, node, _done=None):
-        """Walk on the tree from <node>, getting callbacks from handler."""
-        if _done is None:
-            _done = set()
-        if node in _done:
-            raise AssertionError((id(node), node, node.parent))
-        _done.add(node)
-        self.visit(node)
-        for child_node in node.get_children():
-            assert child_node is not node
-            self.walk(child_node, _done)
-        self.leave(node)
-        assert node.parent is not node
-
-    def get_callbacks(self, node):
+    def get_callbacks(self, node: nodes.NodeNG) -> _CallbackTupleT:
         """Get callbacks from handler for the visited node."""
         klass = node.__class__
         methods = self._cache.get(klass)
         if methods is None:
-            handler = self.handler
             kid = klass.__name__.lower()
             e_method = getattr(
-                handler, f"visit_{kid}", getattr(handler, "visit_default", None)
+                self, f"visit_{kid}", getattr(self, "visit_default", None)
             )
             l_method = getattr(
-                handler, f"leave_{kid}", getattr(handler, "leave_default", None)
+                self, f"leave_{kid}", getattr(self, "leave_default", None)
             )
             self._cache[klass] = (e_method, l_method)
         else:
             e_method, l_method = methods
         return e_method, l_method
 
-    def visit(self, node):
-        """Walk on the tree from <node>, getting callbacks from handler."""
-        method = self.get_callbacks(node)[0]
-        if method is not None:
-            method(node)
-
-    def leave(self, node):
-        """Walk on the tree from <node>, getting callbacks from handler."""
-        method = self.get_callbacks(node)[1]
-        if method is not None:
-            method(node)
-
-
-class LocalsVisitor(ASTWalker):
-    """Visit a project by traversing the locals dictionary."""
-
-    def __init__(self):
-        super().__init__(self)
-        self._visited = set()
-
-    def visit(self, node):
+    def visit(self, node: nodes.NodeNG) -> Any:
         """Launch the visit starting from the given node."""
         if node in self._visited:
             return None
@@ -208,7 +167,7 @@ class LocalsVisitor(ASTWalker):
         return None
 
 
-def get_annotation_label(ann: Union[nodes.Name, nodes.NodeNG]) -> str:
+def get_annotation_label(ann: nodes.Name | nodes.NodeNG) -> str:
     if isinstance(ann, nodes.Name) and ann.name is not None:
         return ann.name
     if isinstance(ann, nodes.NodeNG):
@@ -217,8 +176,8 @@ def get_annotation_label(ann: Union[nodes.Name, nodes.NodeNG]) -> str:
 
 
 def get_annotation(
-    node: Union[nodes.AssignAttr, nodes.AssignName]
-) -> Optional[Union[nodes.Name, nodes.Subscript]]:
+    node: nodes.AssignAttr | nodes.AssignName,
+) -> nodes.Name | nodes.Subscript | None:
     """Return the annotation for `node`."""
     ann = None
     if isinstance(node.parent, nodes.AnnAssign):
@@ -251,15 +210,17 @@ def get_annotation(
     return ann
 
 
-def infer_node(node: Union[nodes.AssignAttr, nodes.AssignName]) -> set:
+def infer_node(node: nodes.AssignAttr | nodes.AssignName) -> set[Any]:
     """Return a set containing the node annotation if it exists
-    otherwise return a set of the inferred types using the NodeNG.infer method
+    otherwise return a set of the inferred types using the NodeNG.infer method.
     """
 
     ann = get_annotation(node)
     try:
         if ann:
-            if isinstance(ann, nodes.Subscript):
+            if isinstance(ann, nodes.Subscript) or (
+                isinstance(ann, nodes.BinOp) and ann.op == "|"
+            ):
                 return {ann}
             return set(ann.infer())
         return set(node.infer())
@@ -267,7 +228,7 @@ def infer_node(node: Union[nodes.AssignAttr, nodes.AssignName]) -> set:
         return {ann} if ann else set()
 
 
-def check_graphviz_availability():
+def check_graphviz_availability() -> None:
     """Check if the ``dot`` command is available on the machine.
 
     This is needed if image output is desired and ``dot`` is used to convert
